/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Preferences Toolbar 2.
 *
 * The Initial Developer of the Original Code is Aaron Andersen.
 *
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Aaron Andersen <aaron@xulplanet.com>
 *                 Kevin Teuscher
 *                 Manuel Reimer <Manuel.Reimer@gmx.de>
 *                 Stephen Clavering <mozilla@clav.me.uk>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var prefbarInFirebird = true;

window.addEventListener("load", prefbarStartPrefBar, true);
window.addEventListener("close", prefbarClosePrefBar, true);


function prefbarStartPrefBar(event)
{
	window.removeEventListener("load", prefbarStartPrefBar, true);
	prefbarCheckForDataFile();

	prefbarLoadDatasource(prefbarGetProfileDir("prefbar.rdf"), "prefbar");

	prefbarCheckForUpdate();

	// the <toolbarset id="customToolbars"/> only exists in browser.xul, not navigator.xul
	prefbarInFirebird = !!document.getElementById("customToolbars");

	// Reload the prefs whenever the window receives focus
	addEventListener("focus", prefbarSetChecks, false);
	// Reload the prefs right now (well, 500ms from now)
	// xxx why not do this right away ??
	setTimeout(prefbarSetChecks, 500);
}


function prefbarClosePrefBar(event) {
  /* Reset the User Agent string pref when exiting the browser completely.

     This is to prevent crashes when using the Java Plugin with the UA set
     to that of IE. See mozilla bug 83376.

     As an added bonus prevents you from forgetting that you changed it and
     sending the wrong UA everywhere for three weeks.
     */

  var windowMediator = Components.classes['@mozilla.org/appshell/window-mediator;1']
                                 .getService(Components.interfaces.nsIWindowMediator);
  var browserWindows = windowMediator.getXULWindowEnumerator("navigator:browser");

  // get the first browser window found.
  var tempwnd = browserWindows.getNext();
  if(browserWindows.getNext()) return; // this is not the final browser window that is closing

  var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                        .getService(Components.interfaces.nsIPrefService);
  var prefBranch = prefs.getBranch("");
  if(prefBranch.prefHasUserValue("general.useragent.override")) {
    prefbarDebugMessage("Clearing spoofed User Agent pref.");
    prefBranch.clearUserPref("general.useragent.override");
  }
}



function prefbarSetChecks()
{
	prefbarDebugMessage("Setting checks...");

	var Rtype    = prefbarRDF.GetResource("http://www.xulplanet.com/rdf/prefbar#type");
	var Renabled = prefbarRDF.GetResource("http://www.xulplanet.com/rdf/prefbar#enabled");

	var Ltrue     = prefbarRDF.GetLiteral("true");
	var Lcheckbox = prefbarRDF.GetLiteral("check");
	var Lbutton   = prefbarRDF.GetLiteral("button");
	var Lmenulist = prefbarRDF.GetLiteral("menulist");
	var Lextlist  = prefbarRDF.GetLiteral("extlist");
	var Lextcheck = prefbarRDF.GetLiteral("extcheck");

	var checkboxes = prefbarMyDatasource.GetSources(Rtype, Lcheckbox, true);

	while(checkboxes.hasMoreElements())
	{
		var box = checkboxes.getNext();
		var id = box.QueryInterface(Components.interfaces.nsIRDFResource).Value;

		if(prefbarMyDatasource.HasAssertion(prefbarRDF.GetResource(id), Renabled, Ltrue, true))
		{
			prefbarSetCheck(id);
		}
	}


  /********* Hack to get rid of ghost buttons that accompany observing buttons ***********/
  /**************** Still doesn't fix the crash on new window problem ********************/
/*
	var buttons = prefbarMyDatasource.GetSources(Rtype, Lbutton, true);

	while(buttons.hasMoreElements())
	{
		var button = buttons.getNext();
		var id = button.QueryInterface(Components.interfaces.nsIRDFResource).Value;

		if(prefbarMyDatasource.HasAssertion(prefbarRDF.GetResource(id), Renabled, Ltrue, true))
     {
			var dialogItem = document.getElementById(id);

			dialogItem.setAttribute("hidden", "true");
			dialogItem.removeAttribute("hidden");
		}
	}
*/


	/*********************** Setup Menulists *********************/

	var menulists = prefbarMyDatasource.GetSources(Rtype, Lmenulist, true);

	while(menulists.hasMoreElements())
	{
		var menulist = menulists.getNext();
		var id = menulist.QueryInterface(Components.interfaces.nsIRDFResource).Value;

		if(prefbarMyDatasource.HasAssertion(prefbarRDF.GetResource(id), Renabled, Ltrue, true))
     		{
			prefbarSetMenulist(id);
		}
	}

	// DVV: simply copying, without understanding, what's going on...
	// Perhaps, I'll get some boogs...
 	var extlists = prefbarMyDatasource.GetSources(Rtype, Lextlist, true);

	while(extlists.hasMoreElements())
	{
		var extlist = extlists.getNext();
		var id = extlist.QueryInterface(Components.interfaces.nsIRDFResource).Value;

		if (prefbarMyDatasource.HasAssertion(prefbarRDF.GetResource(id), Renabled, Ltrue, true))
                {
			prefbarSetExtlist(id);
		}
	}


	var extchecks = prefbarMyDatasource.GetSources(Rtype, Lextcheck, true);

	while(extchecks.hasMoreElements())
	{
		var extcheck = extchecks.getNext();
		var id = extcheck.QueryInterface(Components.interfaces.nsIRDFResource).Value;

		if(prefbarMyDatasource.HasAssertion(prefbarRDF.GetResource(id), Renabled, Ltrue, true))
		{
			prefbarSetExtcheck(id);
		}
	}

}

function prefbarSetCheck(itemId)
{
	var item = document.getElementById(itemId);

	var value = navigator.preference(item.getAttribute("prefstring")); // Value is magic variable referenced in prefstring

	prefbarDebugMessage("Setting Check: " + itemId + " : " + value);

	item.setAttribute("checked",eval(item.getAttribute("frompref")));
}

function prefbarChangePref(event)
{
  var item = event.target;
  var value = item.checked;
  navigator.preference(item.getAttribute("prefstring"),eval(item.getAttribute("topref")));
}



function prefbarSetMenulist(id)
{
	prefbarDebugMessage("Setting menulist: " + id);

	var item = document.getElementById(id);

	var prefstring = item.getAttribute("prefstring");

	var nsIPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService)
	prefBranch = nsIPref.getBranch("");

	if(prefBranch.prefHasUserValue(prefstring))
	{
		var prefvalue = navigator.preference(prefstring);

		prefbarDebugMessage("  Prefvalue: " + prefvalue);

		option = item.firstChild.firstChild;

		do
		{
			var value = option.getAttribute("value");   // Value is magic variable referenced in prefstring
			var listvalue = value; //eval(item.getAttribute("topref"));  // No need for topref in menulist?

			prefbarDebugMessage("  Listvalue: " + listvalue);

			if(prefvalue == listvalue)
			{
				prefbarDebugMessage("Equal!");

				item.selectedIndex = option.getAttribute("position"); // ****** HACK ******
				//item.firstChild.childNodes[option.getAttribute("position")].setAttribute("selected", "true");

/*				var event = document.createEvent('Events');
          event.initEvent('command', false, true);

				item.firstChild.childNodes[option.getAttribute("position")].dispatchEvent(event);
*/				return;
			}
		}	while((option = option.nextSibling));

	}
	else
	{
		item.selectedIndex = item.getAttribute("default");  // ****** HACK ******
		//item.firstChild.childNodes[item.getAttribute("default")].setAttribute("selected", "true");
//		item.firstChild.childNodes[item.getAttribute("default")].dispatchEvent(document.createEvent("click"));
	}

}

function prefbarGoLink(url)
{
	loadURI(url);
}

function prefbarProcessMenulist(item)
{
	var nsIPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService)
	prefBranch = nsIPref.getBranch("");

	var pref = item.getAttribute("prefstring");
	var value = item.selectedItem.getAttribute("value");
	var type = prefBranch.getPrefType(pref);

	if(type == prefBranch.PREF_STRING)
		prefBranch.setCharPref(pref, value);
	else if(type == prefBranch.PREF_INT)
		prefBranch. setIntPref(pref, Number(value));
	else if(type == prefBranch.PREF_BOOL)
		prefBranch.setBoolPref(pref, Boolean(value));
	else
		navigator.preference(pref, value);  // Couldn't hurt to try...


	if(value == "")
	{
		prefBranch.clearUserPref(pref);
	}
}

function prefbarTogglePrefBar() {
  if(prefbarInFirebird) {
    var toolbar = document.getElementById("prefbar");
    toolbar.collapsed = !toolbar.collapsed;
  } else {
    goToggleToolbar('prefbar','viewprefsbar');
  }
  prefbarSetChecks();
}

function prefbarOpenPrefs() {
  if(prefbarInFirebird)
    openDialog('chrome://prefbar/content/preferencesPanel.xul', 'PrefBar', 'chrome,resizable');
  else
    goPreferences('navigator.xul', 'chrome://prefbar/content/preferencesPanel.xul', 'prefbarPanel');
}

/* DVV: Just copy'n'paste from the function above, with some changes... */
function prefbarSetExtlist(id)
{
	prefbarDebugMessage("Setting extlist: " + id);

	var item = document.getElementById(id);

	var getfunction = item.getAttribute("getfunction");

	var value = 0;
	eval(prefbarSingle2Multi(getfunction));

	//if(prefBranch.prefHasUserValue(prefstring))
	if (value != "")
	{
		option = item.firstChild.firstChild;

		do
		{
			var listvalue = option.getAttribute("value");
			//Value is magic variable referenced in get/setfunction

			prefbarDebugMessage("  Value: " + listvalue);

			if (value == listvalue)
			{
				prefbarDebugMessage("Equal!");

				item.selectedIndex = option.getAttribute("position"); // ****** HACK ******
				//item.firstChild.childNodes[option.getAttribute("position")].setAttribute("selected", "true");

/*				var event = document.createEvent('Events');
				event.initEvent('command', false, true);

				item.firstChild.childNodes[option.getAttribute("position")].dispatchEvent(event);
*/				return;
			}
		}	while((option = option.nextSibling));

	}
	else
	{
		item.selectedIndex = item.getAttribute("default");  // ****** HACK ******
		//item.firstChild.childNodes[item.getAttribute("default")].setAttribute("selected", "true");
//		item.firstChild.childNodes[item.getAttribute("default")].dispatchEvent(document.createEvent("click"));
	}

}

/* DVV: just copy'n'paste of the function above, with some changes... */
function prefbarProcessExtlist(item)
{
	var func = item.getAttribute("setfunction");
	var value = item.selectedItem.getAttribute("value");
	eval(prefbarSingle2Multi(func));
	    // function should get a string...
}

function prefbarSetExtcheck(itemId)
{
	var item = document.getElementById(itemId);

	var func = item.getAttribute("getfunction");

	var value;
	eval(prefbarSingle2Multi(func));

	item.setAttribute("checked", value);
}

function prefbarProcessExtcheck(item)
{
	var func = item.getAttribute("setfunction");
  	var value = item.checked;
	eval(prefbarSingle2Multi(func));

	prefbarSetExtcheck(item.getAttribute("id"));
}